package gg.malloc.defense;

import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.plugin.RegisteredServiceProvider;
import org.bukkit.Location;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.World;
import org.bukkit.WorldCreator;

import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.event.EventHandler;

import java.util.logging.Logger;
import java.util.logging.Level;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Collection;

import gg.malloc.defense.model.Arena;
import gg.malloc.defense.model.Waypoint;
import gg.malloc.defense.model.Game;

import gg.malloc.defense.games.LinearGame;
import gg.malloc.defense.games.ScaledWaves;

import gg.malloc.defense.engine.GameRunner;

import gg.malloc.defense.commands.AddPlayerCommand;
import gg.malloc.defense.commands.SetStageCommand;
import gg.malloc.defense.commands.PlayerReadyCommand;
import gg.malloc.defense.commands.ListGamesCommand;
import gg.malloc.defense.commands.JoinGameCommand;
import gg.malloc.defense.commands.LeaveGameCommand;

public class Plugin extends JavaPlugin {
  HashMap<String, Arena> m_arenas = new HashMap<>();
  ArrayList<Game> m_games = new ArrayList<>();
  HashMap<String, GameRunner> m_runningGames = new HashMap<>();
  HashMap<Player, GameRunner> m_playerGames = new HashMap<>();

  public Collection<String> arenaNames() {
    return m_arenas.keySet();
  }

  private class TestSpawn implements Waypoint {
    double m_x;
    double m_y;
    double m_z;
    String m_name;

    public TestSpawn(double x, double y, double z) {
      m_x = x;
      m_y = y;
      m_z = z;
      m_name = "(" + x + "," + y + "," + z + ")";
    }

    public double getX() { return m_x; }
    public double getY() { return m_y; }
    public double getZ() { return m_z; }

    @Override
    public String getName() {
      return m_name;
    }
  }

  public void debuginfo() {
    getLogger().info("Debug Info:");
  }

  @Override
  public void onEnable() {
    getLogger().info("Malloc Defense registered");
    getLogger().setLevel(Level.FINEST);
    //setupDemoGame();
    m_games.add(new ScaledWaves());
    loadArenas();
    getCommand("join").setExecutor(new JoinGameCommand(this));
    getCommand("leave").setExecutor(new LeaveGameCommand(this));
    getCommand("ready").setExecutor(new PlayerReadyCommand(this));
    getCommand("list").setExecutor(new ListGamesCommand(this));

    getCommand("addplayer").setExecutor(new AddPlayerCommand(this));
    getCommand("setstage").setExecutor(new SetStageCommand(this));
    getCommand("debuginfo").setExecutor(new DebuginfoCommand(this));

    getServer().getPluginManager().registerEvents(new PlayerQuitHandler(), this);
  }

  void loadArenas() {
    getLogger().info("Loading arenas...");
    saveDefaultConfig();
    ConfigurationSection pluginConfig = getConfig();
    ConfigurationSection mapList = pluginConfig.getConfigurationSection("maps");

    for(String mapName : mapList.getKeys(false)) {
      getLogger().info("Loading arena: " + mapName);
      ConfigurationSection mapConfig = mapList.getConfigurationSection(mapName);
      List<Map<?, ?>> spawnpointList = mapConfig.getMapList("spawnpoints");
      ArrayList<Waypoint> spawnpoints = new ArrayList<>();

      for(Map<?, ?> spawnerObj : spawnpointList) {
        Map<String, Double> thisSpawner = (Map<String, Double>)spawnerObj;
        double x = thisSpawner.get("x");
        double y = thisSpawner.get("y");
        double z = thisSpawner.get("z");
        spawnpoints.add(new TestSpawn(x, y, z));
      }

      ConfigurationSection targetConfig = mapConfig.getConfigurationSection("target");
      double x = targetConfig.getDouble("x");
      double y = targetConfig.getDouble("y");
      double z = targetConfig.getDouble("z");
      Waypoint bombTarget = new TestSpawn(x, y, z);
      Waypoint[] spawnArray = new Waypoint[spawnpoints.size()];
      spawnpoints.toArray(spawnArray);
      Arena arena = new MemoryArena(spawnArray, bombTarget);
      m_arenas.put(mapName, arena);
    }
  }

  /*GameRunner getRunnerForWorld(World world) {
    GameRunner ret;
    if (m_runningGames.containsKey(world)) {
      ret = m_runningGames.get(world);
    } else if (m_arenas.containsKey(world)) {
      ret = new GameRunner(this, m_games.get(0), m_arenas.get(world.getName()), world);
      m_runningGames.put(world, ret);
      getServer().getPluginManager().registerEvents(new GameEventHandler(ret), this);
    }
    return ret;
  }*/

  public boolean hasRunnerForArenaName(String arenaName) {
    return m_runningGames.containsKey(arenaName);
  }

  public void addPlayerToArena(String arenaName, Player player) {
    GameRunner runner = getRunnerForArenaName(arenaName);
    runner.addPlayer(player);
    m_playerGames.put(player, runner);
  }

  class PlayerQuitHandler implements Listener {
    @EventHandler
    public void onPlayerQuit(PlayerQuitEvent evt) {
      m_playerGames.remove(evt.getPlayer());
    }
  }

  public GameRunner getRunnerForArenaName(String arenaName) {
    GameRunner ret = null;
    if (m_runningGames.containsKey(arenaName)) {
      ret = m_runningGames.get(arenaName);
    } else if (m_arenas.containsKey(arenaName)) {
      getLogger().info("Loading game world " + arenaName);
      World gameWorld = getServer().getWorld(arenaName);
      if (gameWorld == null) {
        getLogger().info("Creating game world " + arenaName);
        gameWorld = new WorldCreator(arenaName).generateStructures(false).createWorld();
      }
      ret = new GameRunner(this, m_games.get(0), m_arenas.get(arenaName), gameWorld);
      getServer().getPluginManager().registerEvents(new GameEventHandler(ret), this);
      m_runningGames.put(arenaName, ret);
      getLogger().info("Game ready for " + arenaName);
    }
    return ret;
  }

  public GameRunner getRunnerForPlayer(Player p) {
    GameRunner ret = null;
    if (m_playerGames.containsKey(p)) {
      ret = m_playerGames.get(p);
    }
    return ret;
  }

  void setupDemoGame() {
    getLogger().info("Setting up demo data'");
    World testWorld = getServer().getWorld("quarry");
    if (testWorld == null) {
      testWorld = new WorldCreator("quarry").generateStructures(false).createWorld();
    }
    Waypoint[] spawnpoints = new Waypoint[3];
    spawnpoints[0] = new TestSpawn(-15, 80, -46);
    spawnpoints[1] = new TestSpawn(-1, 80, -45);
    spawnpoints[2] = new TestSpawn(12, 81, -42);
    Waypoint bombTarget = new TestSpawn(-20, 80, 31);
    m_arenas.put("quarry", new MemoryArena(spawnpoints, bombTarget));
  }
}
